#include <stdio.h>
#include "esc.h"

#ifdef _WIN32
#include <winsock.h>
#include <windows.h>
#define usleep Sleep
#endif
#ifdef __linux__
#include <unistd.h>
#endif

int main() {
#ifdef _WIN32
    WSADATA wsaData;
    WSAStartup(MAKEWORD(2,0), &wsaData);
#endif

    esc_connection_t conn = esc_connection_create(esc_default_connection_settings, "tcp://127.0.0.1:1113", NULL);
    if (conn == 0) {
        return -1;
    }
    if (esc_connection_connect(conn) != 0) {
        return -2;
    }
    esc_credentials_t credentials = esc_credentials_create("admin", "changeit");

    esc_all_events_slice_t result = 0;
    do {
        result = esc_connection_read_all_forward(conn, result ? &result->next_position : NULL, 100, credentials);
        if (result == 0) {
            return -3;
        }
        char posbuf1[44];
        char posbuf2[44];
        printf("%s %s %s %u\n", result->read_direction,
               esc_position_format(&result->from_position, posbuf1, 44),
               esc_position_format(&result->next_position, posbuf2, 44),
               result->is_end_of_stream);
        char uuid_buf[37];
        for (size_t i = 0; i < result->n_events; i++) {
#ifdef _WIN32
            printf("%s %s %lld@%s %llu %llu\n", esc_uuid_format(result->events[i]->event->event_id, uuid_buf, 37),
#else
            printf("%s %s %ld@%s %lu %lu\n", esc_uuid_format(result->events[i]->event->event_id, uuid_buf, 37),
#endif
                   result->events[i]->event->event_type,
                   result->events[i]->event->event_number,
                   result->events[i]->event->event_stream_id,
                   buffer_size(result->events[i]->event->data),
                   buffer_size(result->events[i]->event->metadata));
        }
    } while(result->is_end_of_stream == 0);
	
	usleep(30000);

    esc_connection_close(conn);

#ifdef _WIN32
    WSACleanup();
#endif
    return 0;
}
