//
// Created by nicol on 2018-03-18.
//

#include <memory.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "tcp_package.h"

const uint32_t CommandOffset = 0;
const uint32_t FlagsOffset = 1;
const uint32_t CorrelationOffset = 2;
const uint32_t AuthOffset = 18;
const uint32_t MandatorySize = 18;

tcp_package_t tcp_package_create(uint8_t command, esc_uuid_t correlation_id, buffer_t data) {
    tcp_package_t pkg = malloc(sizeof(struct st_tcp_package));
    pkg->command = command;
    pkg->flags = 0;
    pkg->correlation_id = esc_uuid_copy(correlation_id);
    pkg->login = 0;
    pkg->password = 0;
    pkg->data = data;
    return pkg;
}

tcp_package_t tcp_package_create_authenticated(uint8_t command, esc_uuid_t correlation_id, buffer_t data, const char* username, const char* password) {
    tcp_package_t pkg = malloc(sizeof(struct st_tcp_package));
    pkg->command = command;
    pkg->flags = 1;
    pkg->correlation_id = esc_uuid_copy(correlation_id);
    pkg->login = username;
    pkg->password = password;
    pkg->data = data;
    return pkg;
}

void tcp_package_destroy(tcp_package_t pkg) {
    esc_uuid_destroy(pkg->correlation_id);
    buffer_destroy(pkg->data);
    free(pkg);
}

buffer_t tcp_package_to_buffer(tcp_package_t pkg) {
    size_t data_size = buffer_size(pkg->data);
    uint8_t* buf = malloc(MandatorySize + data_size + (pkg->flags ? 257*2 : 0));
    buf[CommandOffset] = pkg->command;
    buf[FlagsOffset] = pkg->flags;
    memcpy(&buf[CorrelationOffset], pkg->correlation_id, 16);
    size_t size = MandatorySize;
    uint8_t* data = buffer_data(pkg->data);
    if (pkg->flags) {
        size_t l_len = strlen(pkg->login);
        buf[AuthOffset] = (uint8_t)l_len;
        strcpy((char*)&buf[AuthOffset+1], pkg->login);
        size_t p_len = strlen(pkg->password);
        buf[AuthOffset+1+l_len] = (uint8_t)p_len;
        strcpy((char*)&buf[AuthOffset+2+l_len], pkg->password);
        memcpy(&buf[AuthOffset+2+l_len+p_len], data, data_size);
        size += 2 + l_len + p_len + data_size;
    } else {
        memcpy(&buf[AuthOffset], data, data_size);
        size += data_size;
    }
    /*
    for (size_t i = 0; i < size; i++) {
        fprintf(stderr, "%02hhx ", buf[i]);
    }
    fprintf(stderr, "\n");
    */
    return buffer_from(buf, size);
}

tcp_package_t tcp_package_from_buffer(buffer_t buffer) {
    size_t buf_size = buffer_size(buffer);
    uint8_t* buf_data = buffer_data(buffer);
    /*
    for (size_t i = 0; i < buffer_size(buffer); i++) {
        fprintf(stderr, "%02hhx ", buf_data[i]);
    }
    fprintf(stderr, "\n");
    */
    tcp_package_t pkg = malloc(sizeof(struct st_tcp_package));
    pkg->command = buf_data[CommandOffset];
    pkg->flags = buf_data[FlagsOffset];
    pkg->correlation_id = esc_uuid_from(&buf_data[CorrelationOffset], 16);
    if (pkg->flags) {
        size_t l_len = buf_data[AuthOffset];
        size_t p_len = buf_data[AuthOffset+1+l_len];
        size_t pos = MandatorySize - 2 - l_len - p_len;
        pkg->data = buffer_copyfrom(&buf_data[pos], buf_size - pos);
    } else {
        pkg->login = 0;
        pkg->password = 0;
        pkg->data = buffer_copyfrom(&buf_data[MandatorySize], buf_size - MandatorySize);
    }
    return pkg;
}
