//
// Created by nicol on 2018-03-18.
//

#include <stdlib.h>
#include <unistd.h>
#include "socket.h"

#ifdef _WIN32
#include <ws2tcpip.h>
struct st_socket {
    SOCKET s;
    int af;
    int type;
    int proto;
    int last_error;
};

socket_t socket_create(int type) {
    if (type == SOCKET_TYPE_TCP) {
        socket_t s = malloc(sizeof(struct st_socket));
        s->af = AF_INET;
        s->type = SOCK_STREAM;
        s->proto = IPPROTO_TCP;
        s->s = socket(s->af, s->type, s->proto);
        if (s->s == INVALID_SOCKET) {
            s->last_error = WSAGetLastError();
        }
        return s;
    }
    return NULL;
}

int socket_close(socket_t s) {
    int rc = closesocket(s->s);
    if (rc != 0) {
        s->last_error = WSAGetLastError();
    }
    return rc;
}

int socket_connect(socket_t s, char* addr, unsigned short port) {
    char port_s[6];
    ADDRINFO hints;
    memset(&hints, 0, sizeof(ADDRINFO));
    hints.ai_family = s->af;
    hints.ai_socktype = s->type;
    hints.ai_protocol = s->proto;
    ADDRINFO* result;
    int rc;
    if ((rc = getaddrinfo(addr, itoa(port, port_s, 10), &hints, &result)) != 0) {
        s->last_error = WSAGetLastError();
        return -1;
    }
    if ((rc = connect(s->s, result->ai_addr, (int)result->ai_addrlen)) != 0) {
        s->last_error = WSAGetLastError();
        return -2;
    }
    return 0;
}

ssize_t socket_send(socket_t s, char* data, size_t len) {
    ssize_t rc = send(s->s, data, (int)len, 0);
    if (rc < 0) {
        s->last_error = WSAGetLastError();
    }
    return rc;
}

ssize_t socket_recv(socket_t s, char* buf, size_t len) {
    ssize_t rc = recv(s->s, buf, (int)len, 0);
    if (rc < 0) {
        s->last_error = WSAGetLastError();
    }
    return rc;
}

int socket_readable(socket_t s) {
	fd_set readable;
    FD_ZERO(&readable);
	FD_SET(s->s, &readable);
	struct timeval timeout = {0, 0};
	
	int rc = select(0, &readable, 0, 0, &timeout);
    if (rc < 0) {
        s->last_error = WSAGetLastError();
    }
    return rc;
}

int socket_writable(socket_t s) {
    fd_set writable;
    FD_ZERO(&writable);
    FD_SET(s->s, &writable);
    struct timeval timeout = {0, 0};

    int rc = select(0, 0, &writable, 0, &timeout);
    if (rc < 0) {
        s->last_error = WSAGetLastError();
    }
    return rc;
}

int socket_error(socket_t s) {
    return s->last_error;
}
#endif
#ifdef __linux__
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>

struct st_socket {
    int af;
    int type;
    int proto;
    int s;
    int last_error;
};

socket_t socket_create(int type) {
    if (type == SOCKET_TYPE_TCP) {
        socket_t s = malloc(sizeof(struct st_socket));
        s->af = AF_INET;
        s->type = SOCK_STREAM;
        s->proto = IPPROTO_TCP;
        s->s = socket(s->af, s->type, s->proto);
        if (s->s == -1) {
            s->last_error = errno;
        }
        return s;
    }
    return NULL;
}

inline void socket_destroy(socket_t s) {
    socket_close(s);
    free(s);
}

int socket_close(socket_t s) {
    if (close(s->s) == -1) {
        s->last_error = errno;
        return -1;
    }
    return 0;
}

int socket_connect(socket_t s, char* addr, unsigned short port) {
    struct addrinfo hints;
    memset(&hints, 0, sizeof(struct addrinfo));
    hints.ai_family = s->af;
    hints.ai_socktype = s->type;
    hints.ai_protocol = s->proto;
    struct addrinfo* result;
    char port_s[6];
    snprintf(port_s, 6, "%d", port);
    int rc;
    if ((rc = getaddrinfo(addr, port_s, &hints, &result)) != 0) {
        s->last_error = rc;
        return -1;
    }
    if (connect(s->s, result->ai_addr, (int)result->ai_addrlen) != 0) {
        s->last_error = errno;
        return -1;
    }
    return 0;
}

ssize_t socket_send(socket_t s, char* data, size_t len) {
    ssize_t rc = send(s->s, data, len, 0);
    if (rc < 0) {
        s->last_error = errno;
    }
    return rc;
}

ssize_t socket_recv(socket_t s, char* buf, size_t len) {
    ssize_t rc = recv(s->s, buf, len, 0);
    if (rc < 0) {
        s->last_error = errno;
    }
    return rc;
}

int socket_writable(socket_t s) {
    fd_set writable;
    FD_ZERO(&writable);
    FD_SET(s->s, &writable);
    struct timeval timeout = {0,0};

    int rc = select(s->s + 1, 0, &writable, 0, &timeout);
    if (rc < 0) {
        s->last_error = errno;
    }
    return rc;
}

int socket_readable(socket_t s) {
    fd_set readable;
    FD_ZERO(&readable);
    FD_SET(s->s, &readable);
    struct timeval timeout = {0,0};

    int rc = select(s->s + 1, &readable, 0, 0, &timeout);
    if (rc < 0) {
        s->last_error = errno;
    }
    return rc;
}

inline int socket_error(socket_t s) {
    return s->last_error;
}
#endif